# function that calculates the difference between transmission matrix of wall of interest F and transmission matrix
# of equivalent homogenous slab E 

# n_rc = number of resistors and capacitors [-]
# r_tot = total resistance of the layer [C/W]
# c_tot = total capacitance of the layer [J/C]
# p_tot = period of temperature and heat flow fluctuations being considered (12h)




err_FE <- function(n_rc, r_tot, c_tot, p_tot) {
  
  F11 <- 1 + ((2 * pi * r_tot * 1i * c_tot)  / (2 * p_tot * n_rc^2))
  F12 <- (r_tot/n_rc) * (1 + ((2 * pi * r_tot * c_tot *1i)/(4*p_tot*n_rc^2)))
  F21 <- (2 * pi * c_tot * 1i) / (p_tot*n_rc)
  
  
  E11 <- cosh(( (pi * r_tot * c_tot)/p_tot )^(1/2) * (1+1i) )   
  E12 <- sinh( ( (pi * r_tot * c_tot) /p_tot )^(1/2) * (1+1i) ) / (  ((pi * c_tot )/(p_tot * r_tot ))^(1/2) * (1+1i)  )
  E21 <- (sinh( ((pi * r_tot * c_tot)/(p_tot) )^(1/2)* (1+1i)  )) * (((pi * c_tot)/(p_tot * r_tot)  )^(1/2) * (1+1i))
  

  dum_F11 <- c(F11, rep(NA, n_rc-1))
  dum_F12 <- c(F12, rep(NA, n_rc-1))
  dum_F21 <- c(F21, rep(NA, n_rc-1 ))
  
  for(i in 1:n_rc){
    
    dum_F11[i+1] <- dum_F11[i] * dum_F11[1] + dum_F12[i] * dum_F21[1];
    
    dum_F12[i+1] <- dum_F11[i] * dum_F12[1] + dum_F12[i] * dum_F11[1];
    
    dum_F21[i+1] <- dum_F21[i] * dum_F11[1] + dum_F11[i] * dum_F21[1];
    
    
  }
  
 F11 <- dum_F11[n_rc]
 F12 <- dum_F12[n_rc]
 F21 <- dum_F21[n_rc]
 F22 <- F11
  

  
err_F11E11 <- (abs(E11-F11)/abs(E11)) * 100
  
err_F12E12 <- (abs(E12-F12)/abs(E12)) * 100
  
err_F21E21 <- (abs(E21-F21)/abs(E21)) * 100


err_F <- max(err_F11E11, err_F12E12, err_F21E21)

return(err_F)
#return(mat_F)
#return(mat_E)
  
  
}

