# call solver that numerically solves the given ODE for building B05

source(ode_path)
library(ggplot2)
library(deSolve)


######following lines don't need to be adjusted
#set constant input parameters
if (lay_num == 1){
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  
} else if (lay_num == 2){
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                cp_wall2, rho_w2, lambda_wall2, d2,
                
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  
} else if (lay_num == 3){
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                cp_wall2, rho_w2, lambda_wall2, d2,
                cp_wall3, rho_w3, lambda_wall3, d3,
              
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  
} else if (lay_num == 4) {
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                cp_wall2, rho_w2, lambda_wall2, d2,
                cp_wall3, rho_w3, lambda_wall3, d3,
                cp_wall4, rho_w4, lambda_wall4, d4,
                
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  
} else if (lay_num == 5) {
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                cp_wall2, rho_w2, lambda_wall2, d2,
                cp_wall3, rho_w3, lambda_wall3, d3,
                cp_wall4, rho_w4, lambda_wall4, d4,
                cp_wall5, rho_w5, lambda_wall5, d5,
                
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  
} else if (lay_num == 6) {
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                cp_wall2, rho_w2, lambda_wall2, d2,
                cp_wall3, rho_w3, lambda_wall3, d3,
                cp_wall4, rho_w4, lambda_wall4, d4,
                cp_wall5, rho_w5, lambda_wall5, d5,
                cp_wall6, rho_w6, lambda_wall6, d6,
                
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)

  } else if (lay_num == 7) {
  
  parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                cp_wall2, rho_w2, lambda_wall2, d2,
                cp_wall3, rho_w3, lambda_wall3, d3,
                cp_wall4, rho_w4, lambda_wall4, d4,
                cp_wall5, rho_w5, lambda_wall5, d5,
                cp_wall6, rho_w6, lambda_wall6, d6,
                cp_wall7, rho_w7, lambda_wall7, d7,
                
                FF, emis_lw_se, sigma, emis_lw_VGS, 
                rho_a, gamma, cp_air, r_s, LAI, h_wall,
                kin_visc_a, u_a, d_ag, lambda_air, ET_fac)

  } else if (lay_num == 8) {
    
    parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                  cp_wall2, rho_w2, lambda_wall2, d2,
                  cp_wall3, rho_w3, lambda_wall3, d3,
                  cp_wall4, rho_w4, lambda_wall4, d4,
                  cp_wall5, rho_w5, lambda_wall5, d5,
                  cp_wall6, rho_w6, lambda_wall6, d6,
                  cp_wall7, rho_w7, lambda_wall7, d7,
                  cp_wall8, rho_w8, lambda_wall8, d8,
                  
                  FF, emis_lw_se, sigma, emis_lw_VGS, 
                  rho_a, gamma, cp_air, r_s, LAI, h_wall,
                  kin_visc_a, u_a, d_ag, lambda_air, ET_fac)

  } else if (lay_num == 9) {
    
    parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                  cp_wall2, rho_w2, lambda_wall2, d2,
                  cp_wall3, rho_w3, lambda_wall3, d3,
                  cp_wall4, rho_w4, lambda_wall4, d4,
                  cp_wall5, rho_w5, lambda_wall5, d5,
                  cp_wall6, rho_w6, lambda_wall6, d6,
                  cp_wall7, rho_w7, lambda_wall7, d7,
                  cp_wall8, rho_w8, lambda_wall8, d8,
                  cp_wall9, rho_w9, lambda_wall9, d9,
                  
                  FF, emis_lw_se, sigma, emis_lw_VGS, 
                  rho_a, gamma, cp_air, r_s, LAI, h_wall,
                  kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  
    } else if (lay_num == 10) {
    
    parms <- list(cp_wall1, rho_w1, lambda_wall1, d1,
                  cp_wall2, rho_w2, lambda_wall2, d2,
                  cp_wall3, rho_w3, lambda_wall3, d3,
                  cp_wall4, rho_w4, lambda_wall4, d4,
                  cp_wall5, rho_w5, lambda_wall5, d5,
                  cp_wall6, rho_w6, lambda_wall6, d6,
                  cp_wall7, rho_w7, lambda_wall7, d7,
                  cp_wall8, rho_w8, lambda_wall8, d8,
                  cp_wall9, rho_w9, lambda_wall9, d9,
                  cp_wall10, rho_w10, lambda_wall10, d10,
                  
                  FF, emis_lw_se, sigma, emis_lw_VGS, 
                  rho_a, gamma, cp_air, r_s, LAI, h_wall,
                  kin_visc_a, u_a, d_ag, lambda_air, ET_fac)
  }


#set Tamb for required number of consecutive days and expand first value
#to three days (Einschwingen d. DGL)

Tamb <- c(rep(Tair[1], 259200),Tair)

#change Tamb from Celsius to Kelvin
for(i in 1:(length(Tamb))){
        
        Tamb[i] <- Tamb[i] +273.15
}

Tair_in <- c(rep(Tair_i[1], 259200),Tair_i)

#change Tamb from Celsius to Kelvin
for(i in 1:(length(Tair_in))){
        
        Tair_in[i] <- Tair_in[i] +273.15
}


qsol    <- c(rep(q_sol_se[1], 259200),q_sol_se)
qIR     <- c(rep(q_IR_down[1], 259200),q_IR_down)
qsolvgs <- c(rep(q_sol_VGS[1], 259200),q_sol_VGS)
deltab  <- c(rep(delta[1], 259200),delta)
esb     <- c(rep(es[1], 259200),es)
eab     <- c(rep(ea[1], 259200),ea)
r_ab    <- c(rep(r_a[1], 259200),r_a)

input1 <- approxfun(Tamb, method ="constant", rule = 2)
input2 <- approxfun(qsol,  method ="constant",rule = 2)
input3 <- approxfun(qIR,  method ="constant",rule = 2)
input4 <- approxfun(qsolvgs, method ="constant", rule = 2)
input5 <- approxfun(deltab,  method ="constant",rule = 2)
input6 <- approxfun(esb,  method ="constant",rule = 2)
input7 <- approxfun(eab,  method ="constant",rule = 2)
input8 <- approxfun(r_ab,  method ="constant",rule = 2)
input9 <- approxfun(Tair_in,  method ="constant",rule = 2)

#set initial values for all nodes
yini <- rep(Tair[1] + 273.15, nodes_num)
names(yini) <- c(paste(rep("y",nodes_num), seq(1,nodes_num,1), sep=""))

if (greening == 1){
  
  theODE <- t_funcODE
} else if (greening == 0){
  
  theODE <- t_funcODE_blank
} else {
  print("Which ODE should I chose?")
}

# calculate Temperatures (y) at each time step for all 10 nodes 
out <- ode( y = yini,  func = theODE, times = c(seq(0,length(Tamb), 1)), parms = parms)

# turn deSolve-format into data.frame and dont select first 72*3600+1 values
outODE <- as.data.frame(out)
outODE <- outODE[259201:nrow(outODE),]

qin <- h_conv_si * (outODE[, ncol(outODE)] - (273.15 + Tair_i[1]))


outODE <- aggregate(outODE,list(rep(1:(nrow(outODE)%/%n_n+1),each=n_n,len=nrow(outODE))),mean)[-1];

qin <- aggregate(qin,list(rep(1:(length(qin)%/%n_n+1),each=n_n,len=length(qin))),mean)[-1];


Temp_name <- paste("./output/Tnum_", scen_name, ".txt", sep="")
Qin_name <- paste("./output/qin_", scen_name, ".txt", sep="")
 
write.table(outODE, file = Temp_name, row.names = FALSE, col.names = TRUE, sep = "\t", dec =".")
write.table(qin, file = Qin_name, row.names = FALSE, col.names = TRUE, sep = "\t", dec =".")
